@@ -29,6 +29,11 @@ import (
 	"fortio.org/fortio/log"
 )
 
+const (
+	ipOne = "127.0.0.1"
+	ipTwo = "127.0.0.2"
+)
+
 func TestCounter(t *testing.T) {
 	c := NewHistogram(22, 0.1)
 	var b bytes.Buffer
@@ -665,8 +670,12 @@ func TestParsePercentiles(t *testing.T) {
 		// Good cases
 		{str: "99.9", list: []float64{99.9}},
 		{str: "1,2,3", list: []float64{1, 2, 3}},
-		{str: "   17, -5.3,  78  ", list: []float64{17, -5.3, 78}},
+		{str: "   17, 5.3,  78  ", list: []float64{17, 5.3, 78}},
 		// Errors
+		{str: "1024", list: []float64{}, err: true},
+		{str: "0", list: []float64{}, err: true},
+		{str: "100", list: []float64{}, err: true},
+		{str: "-1", list: []float64{}, err: true},
 		{str: "", list: []float64{}, err: true},
 		{str: "   ", list: []float64{}, err: true},
 		{str: "23,a,46", list: []float64{23}, err: true},
@@ -794,6 +803,65 @@ func TestAllBucketBoundaries(t *testing.T) {
 	}
 }
 
+// Unit tests for Occurrence.
+func TestSingleIPOccurrence(t *testing.T) {
+	singleIP := NewOccurrence()
+	totalMap := make(map[string]int)
+	expected := "[127.0.0.1]"
+
+	singleIP.Record(ipOne)
+	singleIP.Record(ipOne)
+
+	actual := singleIP.PrintAndAggregate(totalMap)
+
+	if expected != actual {
+		t.Errorf("Incorrect IP Usage Result. Expected: %s, got: %s", expected, actual)
+	}
+}
+
+func TestMultipleIPOccurrence(t *testing.T) {
+	multiIP := NewOccurrence()
+	totalMap := make(map[string]int)
+	expected := [2]string{"[127.0.0.1 (2), 127.0.0.2 (1)]", "[127.0.0.2 (1), 127.0.0.1 (2)]"}
+
+	multiIP.Record(ipOne)
+	multiIP.Record(ipOne)
+
+	multiIP.Record(ipTwo)
+
+	actual := multiIP.PrintAndAggregate(totalMap)
+
+	if actual != expected[0] && actual != expected[1] {
+		t.Errorf("Incorrect IP Usage Result. Expected: %s, got: %s", expected, actual)
+	}
+}
+
+func TestMultipleConnIPOccurrence(t *testing.T) {
+	occurrenceOne := NewOccurrence()
+	occurrenceTwo := NewOccurrence()
+	totalMap := make(map[string]int)
+
+	// IP usage for first connection.
+	occurrenceOne.Record(ipOne)
+	occurrenceOne.Record(ipOne)
+	occurrenceOne.Record(ipOne)
+	occurrenceOne.Record(ipTwo)
+
+	// IP usage for second connection.
+	occurrenceTwo.Record(ipOne)
+	occurrenceTwo.Record(ipOne)
+	occurrenceTwo.Record(ipTwo)
+	occurrenceTwo.Record(ipTwo)
+
+	_ = occurrenceOne.PrintAndAggregate(totalMap)
+	_ = occurrenceTwo.PrintAndAggregate(totalMap)
+
+	// The occurrence of ip one should be 5 and the occurrence of ip two should be 3.
+	if totalMap[ipOne] != 5 || totalMap[ipTwo] != 3 {
+		t.Errorf("Incorrect total IP usage count.")
+	}
+}
+
 // TODO: add test with data 1.0 1.0001 1.999 2.0 2.5
 // should get 3 buckets 0-1 with count 1
 // 1-2 with count 3
